@@ -69,11 +69,11 @@ def N_cen_ELG_v2(M_h, p_max, logM_cut, sigma, gamma):
         return p_max*(M_h/10**logM_cut)**gamma/(2.5066283*sigma)
 
 @njit(fastmath=True)
-def N_cen_QSO(M_h, p_max, logM_cut, sigma):
+def N_cen_QSO(M_h, logM_cut, sigma):
     """
     HOD function (Zheng et al. (2005) with p_max) for QSO centrals taken from arXiv:2007.09012.
     """
-    return 0.5*p_max*(1 + math.erf((np.log10(M_h)-logM_cut)/1.41421356/sigma))
+    return 0.5*(1 + math.erf((np.log10(M_h)-logM_cut)/1.41421356/sigma))
 
 
 @njit(fastmath=True)
@@ -135,15 +135,14 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
     ic_L, alpha_c_L, Ac_L, Bc_L = LRG_decorations_array[10], LRG_decorations_array[0], \
         LRG_decorations_array[6], LRG_decorations_array[8]
 
-    pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E, A_E = \
+    pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E = \
         ELG_design_array[0], ELG_design_array[1], ELG_design_array[2], ELG_design_array[3], ELG_design_array[4],\
-        ELG_design_array[5], ELG_design_array[6], ELG_design_array[7], ELG_design_array[8]
+        ELG_design_array[5], ELG_design_array[6], ELG_design_array[7]
     alpha_c_E, Ac_E, Bc_E, ic_E = ELG_decorations_array[0], ELG_decorations_array[6], ELG_decorations_array[8],\
     ELG_decorations_array[10]
 
-    pmax_Q, logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q, A_Q = \
-        QSO_design_array[0], QSO_design_array[1], QSO_design_array[2], QSO_design_array[3], QSO_design_array[4],\
-        QSO_design_array[5], QSO_design_array[6]
+    logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q = \
+        QSO_design_array[0], QSO_design_array[1], QSO_design_array[2], QSO_design_array[3], QSO_design_array[4]
     alpha_c_Q, Ac_Q, Bc_Q, ic_Q = QSO_decorations_array[0], QSO_decorations_array[6], QSO_decorations_array[8],\
     QSO_decorations_array[10]
 
@@ -151,7 +150,7 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
 
     numba.set_num_threads(Nthread)
     Nout = np.zeros((Nthread, 3, 8), dtype = np.int64)
-    hstart = np.rint(np.linspace(0, H, Nthread + 1)) # starting index of each thread
+    hstart = np.rint(np.linspace(0, H, Nthread + 1)).astype(np.int64) # starting index of each thread
 
     keep = np.empty(H, dtype = np.int8) # mask array tracking which halos to keep
 
@@ -167,11 +166,11 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
             ELG_marker = LRG_marker
             if want_ELG:
                 logM_cut_E_temp = logM_cut_E + Ac_E * deltac[i] + Bc_E * fenv[i]
-                ELG_marker += N_cen_ELG_v1(mass[i], pmax_E, Q_E, logM_cut_E_temp, sigma_E, gamma_E, A_E) * ic_E * multis[i]
+                ELG_marker += N_cen_ELG_v1(mass[i], pmax_E, Q_E, logM_cut_E_temp, sigma_E, gamma_E) * ic_E * multis[i]
             QSO_marker = ELG_marker
             if want_QSO:
                 logM_cut_Q_temp = logM_cut_Q + Ac_Q * deltac[i] + Bc_Q * fenv[i]
-                QSO_marker += N_cen_QSO(mass[i], pmax_Q, logM_cut_Q, sigma_Q) * ic_Q * multis[i]
+                QSO_marker += N_cen_QSO(mass[i], logM_cut_Q, sigma_Q) * ic_Q * multis[i]
 
             if randoms[i] <= LRG_marker:
                 Nout[tid, 0, 0] += 1 # counting
@@ -238,7 +237,7 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
                 lrg_z[j1] = pos[i,2]
                 lrg_vz[j1] = vel[i,2] + alpha_c_L * vdev[i] # velocity bias
                 # rsd only applies to the z direction
-                if rsd and origin != None:
+                if rsd and origin is not None:
                     nx = lrg_x[j1] - origin[0]
                     ny = lrg_y[j1] - origin[1]
                     nz = lrg_z[j1] - origin[2]
@@ -264,7 +263,7 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
                 elg_z[j2] = pos[i,2]
                 elg_vz[j2] = vel[i,2] + alpha_c_E * vdev[i] # velocity bias
                 # rsd only applies to the z direction
-                if rsd and origin != None:
+                if rsd and origin is not None:
                     nx = elg_x[j2] - origin[0]
                     ny = elg_y[j2] - origin[1]
                     nz = elg_z[j2] - origin[2]
@@ -339,7 +338,7 @@ def gen_cent(pos, vel, mass, ids, multis, randoms, vdev, deltac, fenv,
     QSO_dict['vz'] = qso_vz
     QSO_dict['mass'] = qso_mass
     ID_dict['QSO'] = qso_id
-    return LRG_dict, ELG_dict, QSO_dict, ID_dict
+    return LRG_dict, ELG_dict, QSO_dict, ID_dict, keep
 
 
 @njit(parallel = True, fastmath = True)
@@ -347,7 +346,7 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
     enable_ranks, ranks, ranksv, ranksp, ranksr, 
     LRG_design_array, LRG_decorations_array, ELG_design_array, ELG_decorations_array,
     QSO_design_array, QSO_decorations_array,
-    rsd, inv_velz2kms, lbox, Mpart, want_LRG, want_ELG, want_QSO, Nthread, origin):
+    rsd, inv_velz2kms, lbox, Mpart, want_LRG, want_ELG, want_QSO, Nthread, origin, keep_cent):
 
     """
     Generate satellite galaxies in place in memory with a two pass numba parallel implementation. 
@@ -361,27 +360,26 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
         LRG_decorations_array[5], LRG_decorations_array[6], LRG_decorations_array[7], LRG_decorations_array[8], \
         LRG_decorations_array[9], LRG_decorations_array[10]
 
-    pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E, A_E = \
+    pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E = \
         ELG_design_array[0], ELG_design_array[1], ELG_design_array[2], ELG_design_array[3], ELG_design_array[4],\
-        ELG_design_array[5], ELG_design_array[6], ELG_design_array[7], ELG_design_array[8]
-    alpha_s_E, s_E, s_v_E, s_p_E, s_r_E, Ac_E, As_E, Bc_E, Bs_E, ic_E = \
+        ELG_design_array[5], ELG_design_array[6], ELG_design_array[7]
+    alpha_s_E, s_E, s_v_E, s_p_E, s_r_E, Ac_E, As_E, Bc_E, Bs_E, ic_E, conf_a, conf_c = \
         ELG_decorations_array[1], ELG_decorations_array[2], ELG_decorations_array[3], ELG_decorations_array[4], \
         ELG_decorations_array[5], ELG_decorations_array[6], ELG_decorations_array[7], ELG_decorations_array[8], \
-        ELG_decorations_array[9], ELG_decorations_array[10]
+        ELG_decorations_array[9], ELG_decorations_array[10], ELG_decorations_array[11], ELG_decorations_array[12]
 
-    pmax_Q, logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q, A_Q = \
-        QSO_design_array[0], QSO_design_array[1], QSO_design_array[2], QSO_design_array[3], QSO_design_array[4],\
-        QSO_design_array[5], QSO_design_array[6]
+    logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q = \
+        QSO_design_array[0], QSO_design_array[1], QSO_design_array[2], QSO_design_array[3], QSO_design_array[4]
     alpha_s_Q, s_Q, s_v_Q, s_p_Q, s_r_Q, Ac_Q, As_Q, Bc_Q, Bs_Q, ic_Q = \
         QSO_decorations_array[1], QSO_decorations_array[2], QSO_decorations_array[3], QSO_decorations_array[4], \
         QSO_decorations_array[5], QSO_decorations_array[6], QSO_decorations_array[7], QSO_decorations_array[8], \
         QSO_decorations_array[9], QSO_decorations_array[10]
 
     H = len(hmass) # num of particles
-
+    
     numba.set_num_threads(Nthread)
     Nout = np.zeros((Nthread, 3, 8), dtype = np.int64)
-    hstart = np.rint(np.linspace(0, H, Nthread + 1)) # starting index of each thread
+    hstart = np.rint(np.linspace(0, H, Nthread + 1)).astype(np.int64) # starting index of each thread
 
     keep = np.empty(H, dtype = np.int8) # mask array tracking which halos to keep
 
@@ -404,9 +402,16 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
             ELG_marker = LRG_marker
             if want_ELG:
                 M1_E_temp = 10**(logM1_E + As_E * hdeltac[i] + Bs_E * hfenv[i])
+                # elg conformity
+                if keep_cent[i] == 1:
+                    M1_E_temp = M1_E_temp*10**conf_c
+                elif keep_cent[i] == 2:
+                    M1_E_temp = M1_E_temp*10**conf_a
                 logM_cut_E_temp = logM_cut_E + Ac_E * hdeltac[i] + Bc_E * hfenv[i]
                 base_p_E = N_sat_generic(
                     hmass[i], 10**logM_cut_E_temp, kappa_E, M1_E_temp, alpha_E) * weights[i] * ic_E
+                # print(base_p_E, np.log10(hmass[i]), N_sat_generic(
+                # hmass[i], 10**logM_cut_E_temp, kappa_E, M1_E_temp, alpha_E), weights[i], ic_E)
                 if enable_ranks:
                     decorator_E = 1 + s_E * ranks[i] + s_v_E * ranksv[i] + s_p_E * ranksp[i] + s_r_E * ranksr[i]
                     exp_sat = base_p_E * decorator_E
@@ -418,7 +423,7 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
                 M1_Q_temp = 10**(logM1_Q + As_Q * hdeltac[i] + Bs_Q * hfenv[i])
                 logM_cut_Q_temp = logM_cut_Q + Ac_Q * hdeltac[i] + Bc_Q * hfenv[i]
                 base_p_Q = N_sat_generic(
-                    hmass[i], 10**logM_cut_Q_temp, kappa_Q, M1_Q_temp, alpha_Q, A_Q) * weights[i] * ic_Q
+                    hmass[i], 10**logM_cut_Q_temp, kappa_Q, M1_Q_temp, alpha_Q) * weights[i] * ic_Q
                 if enable_ranks:
                     decorator_Q = 1 + s_Q * ranks[i] + s_v_Q * ranksv[i] + s_p_Q * ranksp[i] + s_r_Q * ranksr[i]
                     exp_sat = base_p_Q * decorator_Q
@@ -489,7 +494,7 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
                 lrg_vy[j1] = hvel[i, 1] + alpha_s_L * (pvel[i, 1] - hvel[i, 1]) # velocity bias
                 lrg_z[j1] = ppos[i, 2]
                 lrg_vz[j1] = hvel[i, 2] + alpha_s_L * (pvel[i, 2] - hvel[i, 2]) # velocity bias
-                if rsd and origin != None:
+                if rsd and origin is not None:
                     nx = lrg_x[j1] - origin[0]
                     ny = lrg_y[j1] - origin[1]
                     nz = lrg_z[j1] - origin[2]
@@ -513,7 +518,7 @@ def gen_sats(ppos, pvel, hvel, hmass, hid, weights, randoms, hdeltac, hfenv,
                 elg_vy[j2] = hvel[i, 1] + alpha_s_E * (pvel[i, 1] - hvel[i, 1]) # velocity bias
                 elg_z[j2] = ppos[i, 2]
                 elg_vz[j2] = hvel[i, 2] + alpha_s_E * (pvel[i, 2] - hvel[i, 2]) # velocity bias
-                if rsd and origin != None:
+                if rsd and origin is not None:
                     nx = elg_x[j2] - origin[0]
                     ny = elg_y[j2] - origin[1]
                     nz = elg_z[j2] - origin[2]
@@ -611,8 +616,8 @@ def fast_concatenate(array1, array2, Nthread):
     numba.set_num_threads(Nthread)
     Nthread1 = max(1, int(np.floor(Nthread * N1 / (N1 + N2))))
     Nthread2 = Nthread - Nthread1
-    hstart1 = np.rint(np.linspace(0, N1, Nthread1 + 1))
-    hstart2 = np.rint(np.linspace(0, N2, Nthread2 + 1)) + N1
+    hstart1 = np.rint(np.linspace(0, N1, Nthread1 + 1)).astype(np.int64)
+    hstart2 = np.rint(np.linspace(0, N2, Nthread2 + 1)).astype(np.int64) + N1
 
     for tid in numba.prange(Nthread): #numba.prange(Nthread):
         if tid < Nthread1:
@@ -624,6 +629,7 @@ def fast_concatenate(array1, array2, Nthread):
     # final_array = np.concatenate((array1, array2))
     return final_array
 
+
 def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd, verbose):
     """
     parse hod parameters, pass them on to central and satellite generators 
@@ -671,19 +677,19 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
                               'alpha', 
                               'kappa'))
         LRG_design_array = np.array([logM_cut_L, logM1_L, sigma_L, alpha_L, kappa_L])
+        
+        alpha_c = LRG_HOD.get('alpha_c', 0)
+        alpha_s = LRG_HOD.get('alpha_s', 1)
+        s = LRG_HOD.get('s', 0)
+        s_p = LRG_HOD.get('s_p', 0)
+        s_v = LRG_HOD.get('s_v', 0)
+        s_r = LRG_HOD.get('s_r', 0)
+        Ac = LRG_HOD.get('Acent', 0)
+        As = LRG_HOD.get('Asat', 0)
+        Bc = LRG_HOD.get('Bcent', 0)
+        Bs = LRG_HOD.get('Bsat', 0)
+        ic = LRG_HOD.get('ic', 1)
 
-        alpha_c, alpha_s, s, s_v, s_p, s_r, Ac, As, Bc, Bs, ic = \
-            map(LRG_HOD.get, ('alpha_c', 
-                            'alpha_s',  
-                            's', 
-                            's_v', 
-                            's_p', 
-                            's_r',
-                            'Acent',
-                            'Asat',
-                            'Bcent',
-                            'Bsat',
-                            'ic'))
         LRG_decorations_array = np.array([alpha_c, alpha_s, s, s_v, s_p, s_r, Ac, As, Bc, Bs, ic])
     else:
         # B.H. TODO: this will go when we switch to dictionaried and for loops
@@ -694,7 +700,7 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
     if 'ELG' in tracers.keys():
         # ELG design
         want_ELG = True
-        pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E, A_E = \
+        pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E = \
             map(ELG_HOD.get, ('p_max',
                             'Q',
                             'logM_cut',
@@ -702,61 +708,64 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
                             'sigma',
                             'logM1',
                             'alpha',
-                            'gamma',
-                            'A_s'))
+                            'gamma'))
         ELG_design_array = np.array(
-            [pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E, A_E])
-        alpha_c_E, alpha_s_E, s_E, s_v_E, s_p_E, s_r_E, Ac_E, As_E, Bc_E, Bs_E, ic_E = \
-            map(ELG_HOD.get, ('alpha_c', 
-                            'alpha_s',  
-                            's', 
-                            's_v', 
-                            's_p', 
-                            's_r',
-                            'Acent',
-                            'Asat',
-                            'Bcent',
-                            'Bsat',
-                            'ic'))
+            [pmax_E, Q_E, logM_cut_E, kappa_E, sigma_E, logM1_E, alpha_E, gamma_E])
+        
+        alpha_c_E = ELG_HOD.get('alpha_c', 0)
+        alpha_s_E = ELG_HOD.get('alpha_s', 1)
+        s_E = ELG_HOD.get('s', 0)
+        s_p_E = ELG_HOD.get('s_p', 0)
+        s_v_E = ELG_HOD.get('s_v', 0)
+        s_r_E = ELG_HOD.get('s_r', 0)
+        Ac_E = ELG_HOD.get('Acent', 0)
+        As_E = ELG_HOD.get('Asat', 0)
+        Bc_E = ELG_HOD.get('Bcent', 0)
+        Bs_E = ELG_HOD.get('Bsat', 0)
+        ic_E = ELG_HOD.get('ic', 1)
+        
+        # conformity params
+        conf_a = ELG_HOD.get('conf_a', 0)
+        conf_c = ELG_HOD.get('conf_c', 0)
+        
         ELG_decorations_array = np.array(
-            [alpha_c_E, alpha_s_E, s_E, s_v_E, s_p_E, s_r_E, Ac_E, As_E, Bc_E, Bs_E, ic_E])
+            [alpha_c_E, alpha_s_E, s_E, s_v_E, s_p_E, s_r_E, Ac_E, As_E, Bc_E, Bs_E, ic_E, conf_a, conf_c])
     else:
         # B.H. TODO: this will go when we switch to dictionaried and for loops
-        ELG_design_array = np.zeros(9)
-        ELG_decorations_array = np.zeros(10)
+        ELG_design_array = np.zeros(8)
+        ELG_decorations_array = np.zeros(11)
         want_ELG = False
         
     if 'QSO' in tracers.keys():
         # QSO design
         want_QSO = True
-        pmax_Q, logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q, A_Q = \
-            map(QSO_HOD.get, ('p_max',
-                            'logM_cut',
+        logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q = \
+            map(QSO_HOD.get, ('logM_cut',
                             'kappa',
                             'sigma',
                             'logM1',
-                            'alpha',
-                            'A_s'))
+                            'alpha'))
         QSO_design_array = np.array(
-            [pmax_Q, logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q, A_Q])
-        alpha_c_Q, alpha_s_Q, s_Q, s_v_Q, s_p_Q, s_r_Q, Ac_Q, As_Q, Bc_Q, Bs_Q, ic_Q = \
-            map(QSO_HOD.get, ('alpha_c', 
-                            'alpha_s',  
-                            's', 
-                            's_v', 
-                            's_p', 
-                            's_r',
-                            'Acent',
-                            'Asat',
-                            'Bcent',
-                            'Bsat',
-                            'ic'))
+            [logM_cut_Q, kappa_Q, sigma_Q, logM1_Q, alpha_Q])
+        
+        alpha_c_Q = QSO_HOD.get('alpha_c', 0)
+        alpha_s_Q = QSO_HOD.get('alpha_s', 1)
+        s_Q = QSO_HOD.get('s', 0)
+        s_p_Q = QSO_HOD.get('s_p', 0)
+        s_v_Q = QSO_HOD.get('s_v', 0)
+        s_r_Q = QSO_HOD.get('s_r', 0)
+        Ac_Q = QSO_HOD.get('Acent', 0)
+        As_Q = QSO_HOD.get('Asat', 0)
+        Bc_Q = QSO_HOD.get('Bcent', 0)
+        Bs_Q = QSO_HOD.get('Bsat', 0)
+        ic_Q = QSO_HOD.get('ic', 1)
+        
         QSO_decorations_array = np.array(
             [alpha_c_Q, alpha_s_Q, s_Q, s_v_Q, s_p_Q, s_r_Q, Ac_Q, As_Q, Bc_Q, Bs_Q, ic_Q])
     else:
         # B.H. TODO: this will go when we switch to dictionaried and for loops
-        QSO_design_array = np.zeros(7)
-        QSO_decorations_array = np.zeros(10)
+        QSO_design_array = np.zeros(5)
+        QSO_decorations_array = np.zeros(11)
         want_QSO = False
     
     start = time.time()
@@ -766,14 +775,22 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
     lbox = params['Lbox']
     origin = params['origin']
     # for each halo, generate central galaxies and output to file
-    LRG_dict_cent, ELG_dict_cent, QSO_dict_cent, ID_dict_cent = \
+    LRG_dict_cent, ELG_dict_cent, QSO_dict_cent, ID_dict_cent, keep_cent = \
     gen_cent(halos_array['hpos'], halos_array['hvel'], halos_array['hmass'], halos_array['hid'], halos_array['hmultis'], 
              halos_array['hrandoms'], halos_array['hveldev'], halos_array['hdeltac'], halos_array['hfenv'], 
              LRG_design_array, LRG_decorations_array, ELG_design_array, ELG_decorations_array, QSO_design_array, 
              QSO_decorations_array, rsd, inv_velz2kms, lbox, want_LRG, want_ELG, want_QSO, Nthread, origin)
     if verbose:
         print("generating centrals took ", time.time() - start)
-
+    
+    # if conf_a or conf_c: 
+    #     start = time.time()
+    #     assert np.all(halos_array['hid'][:-1] <= halos_array['hid'][1:])
+    #     # pkeeps = fast_keep(keep_cent, halos_array['hid'], subsample['phid'])
+    #     pinds = searchsorted_parallel(halos_array['hid'], subsample['phid'], Nthread)
+    #     pkeeps = keep_cent[subsample['pinds']]
+    #     print("gen pkeep took", time.time() - start)
+        
 
     start = time.time()
     LRG_dict_sat, ELG_dict_sat, QSO_dict_sat, ID_dict_sat = \
@@ -782,7 +799,7 @@ def gen_gals(halos_array, subsample, tracers, params, Nthread, enable_ranks, rsd
              enable_ranks, subsample['pranks'], subsample['pranksv'], subsample['pranksp'], subsample['pranksr'],
              LRG_design_array, LRG_decorations_array, ELG_design_array, ELG_decorations_array,
              QSO_design_array, QSO_decorations_array, rsd, inv_velz2kms, lbox, params['Mpart'],
-             want_LRG, want_ELG, want_QSO, Nthread, origin)
+             want_LRG, want_ELG, want_QSO, Nthread, origin, keep_cent[subsample['pinds']])
     if verbose:
         print("generating satellites took ", time.time() - start)
 
@@ -887,6 +904,9 @@ def gen_gal_cat(halo_data, particle_data, tracers, params, Nthread = 16,
             # save to file 
             outdict = HOD_dict[tracer].pop('Ncent', None)
             table = Table(HOD_dict[tracer], meta = {'Ncent': Ncent, 'Gal_type': tracer, **tracers[tracer]})
-            ascii.write(table, outdir / ("%ss.dat"%tracer), overwrite = True, format = 'ecsv')
+            if params['chunk'] == -1:
+                ascii.write(table, outdir / (f"{tracer}s.dat"), overwrite = True, format = 'ecsv')
+            else:
+                ascii.write(table, outdir / (f"{tracer}s_chunk{params['chunk']:d}.dat"), overwrite = True, format = 'ecsv')
 
     return HOD_dict
