@@ -92,7 +92,7 @@ where the central velocity bias parameter :math:`\\alpha_c` sets the ratio of
 central velocity dispersion vs. halo velocity dispersion. The satellite 
 velocity bias parameter :math:`\\alpha_c` sets the ratio between the satellite
 peculiar velocity to the particle peculiar velocity. The default for these two
-parameters are 1. 
+parameters are 0 and 1, respectively. 
 
 We additionaly introduce a set of satellite profile parameters 
 :math:`(s, s_v, s_p, s_r)` that allow for flexibilities in how satellite 
@@ -225,6 +225,9 @@ import time
 import timeit
 from pathlib import Path
 
+import numba
+from numba import njit
+
 import numpy as np
 import h5py
 import asdf
@@ -238,11 +241,18 @@ from .parallel_numpy_rng import *
 from .tpcf_corrfunc import calc_xirppi_fast, calc_wp_fast, calc_multipole_fast
 # TODO B.H.: staging can be shorter and prettier; perhaps asdf for h5 and ecsv?
 
+@njit(parallel=True)
+def searchsorted_parallel(a, b):
+    res = np.empty(len(b), dtype = np.int64)
+    for i in numba.prange(len(b)):
+        res[i] = np.searchsorted(a, b[i])
+    return res
+
 class AbacusHOD:
     """
     A highly efficient multi-tracer HOD code for the AbacusSummmit simulations.
     """
-    def __init__(self, sim_params, HOD_params, clustering_params):
+    def __init__(self, sim_params, HOD_params, clustering_params = None, chunk=-1, n_chunks=1):
         """
         Loads simulation. The ``sim_params`` dictionary specifies which simulation
         volume to load. The ``HOD_params`` specifies the HOD parameters and tracer
@@ -276,7 +286,7 @@ class AbacusHOD:
                 * ``ELG_params``: dict, HOD parameter values for ELGs. Default values are given in config file. 
                 * ``QSO_params``: dict, HOD parameter values for QSOs. Default values are given in config file. 
 
-        clustering_params: dict
+        clustering_params: dict, optional
             Summary statistics configuration parameters. Load from ``config/abacus_hod.yaml``. It contains the following keys:
                 * ``clustering_type``: str, which summary statistic to compute. Options: ``wp``, ``xirppi``, default: ``xirppi``.
                 * ``bin_params``: dict, transverse scale binning. 
@@ -285,14 +295,17 @@ class AbacusHOD:
                     * ``nbins``: int, number of bins.
                 * ``pimax``: int, :math:`\\pi_{\\mathrm{max}}`. 
                 * ``pi_bin_size``: int, size of bins along of the line of sight. Need to be divisor of ``pimax``.
-
+        chunk: int, optional
+            Index of current chunk. Must be between ``0`` and ``n_chunks-1``. Files associated with this chunk are written out as ``{tracer}s_{chunk}.dat``. Default is -1 (no chunking).
+        n_chunks: int, optional
+            Number of chunks to split the input from the halo+particle subsample and number of output files in which to write out the galaxy catalogs following the format ``{tracer}s_{chunk}.dat``.
         """
         # simulation details
         self.sim_name = sim_params['sim_name']
         self.sim_dir = sim_params['sim_dir']
         self.subsample_dir = sim_params['subsample_dir']
         self.z_mock = sim_params['z_mock']
-        self.output_dir = sim_params['output_dir']
+        self.output_dir = sim_params.get('output_dir', './')
         self.halo_lc = sim_params.get('halo_lc', False)
         
         # tracers
@@ -304,16 +317,22 @@ class AbacusHOD:
         self.tracers = tracers
 
         # HOD parameter choices
-        self.want_ranks = HOD_params['want_ranks']
+        self.want_ranks = HOD_params.get('want_ranks', False)
         self.want_rsd = HOD_params['want_rsd']
+        
+        if not clustering_params == None:
+            # clusteringparameters
+            self.pimax = clustering_params.get('pimax', None)
+            self.pi_bin_size = clustering_params.get('pi_bin_size', None)
+            bin_params = clustering_params['bin_params']
+            self.rpbins = np.logspace(bin_params['logmin'], bin_params['logmax'], bin_params['nbins'] + 1)
+            self.clustering_type = clustering_params.get('clustering_type', None)
 
-        # clusteringparameters
-        self.pimax = clustering_params['pimax']
-        self.pi_bin_size = clustering_params['pi_bin_size']
-        bin_params = clustering_params['bin_params']
-        self.rpbins = np.logspace(bin_params['logmin'], bin_params['logmax'], bin_params['nbins'] + 1)
-        self.clustering_type = clustering_params['clustering_type']
-                
+        # setting up chunking
+        self.chunk = chunk
+        self.n_chunks = n_chunks
+        assert self.chunk < self.n_chunks, "Total number of chunks needs to be larger than current chunk index"
+        
         # load the subsample particles
         self.halo_data, self.particle_data, self.params, self.mock_dir = self.staging()
 
@@ -364,14 +383,25 @@ class AbacusHOD:
             params['origin'] = np.array(header['LightConeOrigins']).reshape(-1,3)[0]
         else:
             params['origin'] = None # observer at infinity in the -z direction
-        params['numslabs'] = len(halo_info_fns)
+
+        # settitng up chunking
+        n_chunks = self.n_chunks
+        params['chunk'] = self.chunk
+        if self.chunk == -1:
+            chunk = 0
+        else:
+            chunk = self.chunk
+        n_jump = int(np.ceil(len(halo_info_fns)/n_chunks))
+        start = ((chunk)*n_jump)
+        end = ((chunk+1)*n_jump)
+        if end > len(halo_info_fns): end = len(halo_info_fns)
+        params['numslabs'] = end-start
         self.lbox = header['BoxSize']
 
         # count ther number of halos and particles
         Nhalos = np.empty(params['numslabs'])
         Nparts = np.empty(params['numslabs'])        
-        for eslab in range(params['numslabs']):
-            
+        for eslab in range(start, end):
             if 'ELG' not in self.tracers.keys() and 'QSO' not in self.tracers.keys():
                 halofilename = subsample_dir / ('halos_xcom_%d_seed600_abacushod_oldfenv'%eslab)
                 particlefilename = subsample_dir / ('particles_xcom_%d_seed600_abacushod_oldfenv'%eslab)
@@ -386,8 +416,10 @@ class AbacusHOD:
 
             newfile = h5py.File(halofilename, 'r')
             newpart = h5py.File(particlefilename, 'r')
-            Nhalos[eslab] = len(newfile['halos'])
-            Nparts[eslab] = len(newpart['particles'])
+
+            Nhalos[eslab-start] = len(newfile['halos'])
+            Nparts[eslab-start] = len(newpart['particles'])
+
         Nhalos = Nhalos.astype(int)
         Nparts = Nparts.astype(int)
         Nhalos_tot = int(np.sum(Nhalos))
@@ -427,7 +459,8 @@ class AbacusHOD:
         # load all the halo and particle data we need
         halo_ticker = 0
         parts_ticker = 0
-        for eslab in range(params['numslabs']):
+        for eslab in range(start, end):
+            
             print("Loading simulation by slab, ", eslab)
             
             if 'ELG' not in self.tracers.keys() and 'QSO' not in self.tracers.keys():
@@ -460,18 +493,18 @@ class AbacusHOD:
             halo_submask = maskedhalos['mask_subsample'].astype(bool)
             halo_randoms = maskedhalos['randoms']
 
-            hpos[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_pos
-            hvel[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_vels
-            hmass[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_mass
-            hid[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_ids
-            hmultis[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_multi
-            hrandoms[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_randoms
-            hveldev[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_vel_dev
-            hsigma3d[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_sigma3d
-            hdeltac[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_deltac
-            hfenv[halo_ticker: halo_ticker + Nhalos[eslab]] = halo_fenv
-            halo_ticker += Nhalos[eslab]
-
+            hpos[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_pos
+            hvel[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_vels
+            hmass[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_mass
+            hid[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_ids
+            hmultis[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_multi
+            hrandoms[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_randoms
+            hveldev[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_vel_dev
+            hsigma3d[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_sigma3d
+            hdeltac[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_deltac
+            hfenv[halo_ticker: halo_ticker + Nhalos[eslab-start]] = halo_fenv
+            halo_ticker += Nhalos[eslab-start]
+            
             # extract particle data that we need
             newpart = h5py.File(particlefilename, 'r')
             subsample = newpart['particles']
@@ -491,25 +524,26 @@ class AbacusHOD:
                 part_ranksv = subsample['ranksv']
                 part_ranksp = subsample['ranksp']
                 part_ranksr = subsample['ranksr']
-                p_ranks[parts_ticker: parts_ticker + Nparts[eslab]] = part_ranks
-                p_ranksv[parts_ticker: parts_ticker + Nparts[eslab]] = part_ranksv
-                p_ranksp[parts_ticker: parts_ticker + Nparts[eslab]] = part_ranksp
-                p_ranksr[parts_ticker: parts_ticker + Nparts[eslab]] = part_ranksr
 
+                p_ranks[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_ranks
+                p_ranksv[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_ranksv
+                p_ranksp[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_ranksp
+                p_ranksr[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_ranksr
+                
             # #     part_data_slab += [part_ranks, part_ranksv, part_ranksp, part_ranksr]
             # particle_data = vstack([particle_data, new_part_table])
-            ppos[parts_ticker: parts_ticker + Nparts[eslab]] = part_pos
-            pvel[parts_ticker: parts_ticker + Nparts[eslab]] = part_vel
-            phvel[parts_ticker: parts_ticker + Nparts[eslab]] =  part_hvel
-            phmass[parts_ticker: parts_ticker + Nparts[eslab]] = part_halomass
-            phid[parts_ticker: parts_ticker + Nparts[eslab]] =  part_haloid
-            pNp[parts_ticker: parts_ticker + Nparts[eslab]] = part_Np
-            psubsampling[parts_ticker: parts_ticker + Nparts[eslab]] = part_subsample
-            prandoms[parts_ticker: parts_ticker + Nparts[eslab]] = part_randoms
-            pdeltac[parts_ticker: parts_ticker + Nparts[eslab]] = part_deltac
-            pfenv[parts_ticker: parts_ticker + Nparts[eslab]] = part_fenv
-            parts_ticker += Nparts[eslab]
-
+            ppos[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_pos
+            pvel[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_vel
+            phvel[parts_ticker: parts_ticker + Nparts[eslab-start]] =  part_hvel
+            phmass[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_halomass
+            phid[parts_ticker: parts_ticker + Nparts[eslab-start]] =  part_haloid
+            pNp[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_Np
+            psubsampling[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_subsample
+            prandoms[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_randoms
+            pdeltac[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_deltac
+            pfenv[parts_ticker: parts_ticker + Nparts[eslab-start]] = part_fenv
+            parts_ticker += Nparts[eslab-start]
+            
         halo_data = {"hpos": hpos, 
                      "hvel": hvel, 
                      "hmass": hmass, 
@@ -521,6 +555,9 @@ class AbacusHOD:
                      "hdeltac": hdeltac, 
                      "hfenv": hfenv}
         pweights = 1/pNp/psubsampling
+        # index from particles to halos, for conformity
+        assert np.all(hid[:-1] <= hid[1:])
+        pinds = searchsorted_parallel(hid, phid)
         particle_data = {"ppos": ppos, 
                          "pvel": pvel, 
                          "phvel": phvel, 
@@ -529,7 +566,8 @@ class AbacusHOD:
                          "pweights": pweights, 
                          "prandoms": prandoms, 
                          "pdeltac": pdeltac, 
-                         "pfenv": pfenv}
+                         "pfenv": pfenv,
+                         "pinds": pinds}
         if self.want_ranks:
             particle_data['pranks'] = p_ranks
             particle_data['pranksv'] = p_ranksv
@@ -609,7 +647,7 @@ class AbacusHOD:
             rsd = want_rsd, 
             write_to_disk = write_to_disk, 
             savedir = self.mock_dir,
-            verbose = False,
+            verbose = verbose,
             fn_ext = fn_ext)
         print("gen mocks", time.time() - start)
 
@@ -649,8 +687,8 @@ class AbacusHOD:
                 newngal = AbacusHOD._compute_ngal_lrg(
                     self.logMbins, self.deltacbins, self.fenvbins, self.halo_mass_func,
                     tracer_hod['logM_cut'], tracer_hod['logM1'], tracer_hod['sigma'], 
-                    tracer_hod['alpha'], tracer_hod['kappa'], tracer_hod['Acent'], 
-                    tracer_hod['Asat'], tracer_hod['Bcent'], tracer_hod['Bsat'], tracer_hod['ic'], Nthread)
+                    tracer_hod['alpha'], tracer_hod['kappa'], tracer_hod.get('Acent', 0), 
+                    tracer_hod.get('Asat', 0), tracer_hod.get('Bcent', 0), tracer_hod.get('Bsat', 0), tracer_hod.get('ic', 1), Nthread)
                 ngal_dict[etracer] = newngal[0] + newngal[1]
                 fsat_dict[etracer] = newngal[1] / (newngal[0] + newngal[1])
             elif etracer == 'ELG':
@@ -658,17 +696,17 @@ class AbacusHOD:
                     self.logMbins, self.deltacbins, self.fenvbins, self.halo_mass_func, 
                     tracer_hod['p_max'], tracer_hod['Q'], tracer_hod['logM_cut'], 
                     tracer_hod['kappa'], tracer_hod['sigma'], tracer_hod['logM1'],  
-                    tracer_hod['alpha'], tracer_hod['gamma'], tracer_hod['A_s'],  
-                    tracer_hod['Acent'],  tracer_hod['Asat'],  tracer_hod['Bcent'],  tracer_hod['Bsat'], tracer_hod['ic'], Nthread) 
+                    tracer_hod['alpha'], tracer_hod['gamma'], tracer_hod.get('Acent', 0), 
+                    tracer_hod.get('Asat', 0), tracer_hod.get('Bcent', 0), tracer_hod.get('Bsat', 0), tracer_hod.get('ic', 1), Nthread) 
                 ngal_dict[etracer] = newngal[0] + newngal[1]
                 fsat_dict[etracer] = newngal[1] / (newngal[0] + newngal[1])
             elif etracer == 'QSO':
                 newngal = AbacusHOD._compute_ngal_qso(
                     self.logMbins, self.deltacbins, self.fenvbins, self.halo_mass_func, 
-                    tracer_hod['p_max'], tracer_hod['logM_cut'], 
+                    tracer_hod['logM_cut'], 
                     tracer_hod['kappa'], tracer_hod['sigma'], tracer_hod['logM1'],  
-                    tracer_hod['alpha'], tracer_hod['A_s'],  
-                    tracer_hod['Acent'],  tracer_hod['Asat'],  tracer_hod['Bcent'],  tracer_hod['Bsat'], tracer_hod['ic'], Nthread)         
+                    tracer_hod['alpha'], tracer_hod.get('Acent', 0), 
+                    tracer_hod.get('Asat', 0), tracer_hod.get('Bcent', 0), tracer_hod.get('Bsat', 0), tracer_hod.get('ic', 1), Nthread)         
                 ngal_dict[etracer] = newngal[0] + newngal[1]
                 fsat_dict[etracer] = newngal[1] / (newngal[0] + newngal[1])
         return ngal_dict, fsat_dict
@@ -703,7 +741,7 @@ class AbacusHOD:
     @staticmethod
     @njit(fastmath = True, parallel = True)
     def _compute_ngal_elg(logMbins, deltacbins, fenvbins, halo_mass_func, p_max, Q,
-                   logM_cut, kappa, sigma, logM1, alpha, gamma, A_s, Acent, Asat, Bcent, Bsat, ic, Nthread):
+                   logM_cut, kappa, sigma, logM1, alpha, gamma, Acent, Asat, Bcent, Bsat, ic, Nthread):
         """
         internal helper to compute number of LRGs
         """
@@ -720,7 +758,7 @@ class AbacusHOD:
                     Mh_temp = 10**logMs[i]
                     logM_cut_temp = logM_cut + Acent * deltacs[j] + Bcent * fenvs[k]
                     M1_temp = 10**(logM1 + Asat * deltacs[j] + Bsat * fenvs[k])
-                    ncent_temp = N_cen_ELG_v1(Mh_temp, p_max, Q, logM_cut_temp, sigma, gamma, A_s)
+                    ncent_temp = N_cen_ELG_v1(Mh_temp, p_max, Q, logM_cut_temp, sigma, gamma)
                     nsat_temp = N_sat_generic(Mh_temp, 10**logM_cut_temp, kappa, M1_temp, alpha)
                     ngal_cent += halo_mass_func[i, j, k] * ncent_temp * ic
                     ngal_sat += halo_mass_func[i, j, k] * nsat_temp * ic
@@ -728,8 +766,8 @@ class AbacusHOD:
 
     @staticmethod
     @njit(fastmath = True, parallel = True)
-    def _compute_ngal_qso(logMbins, deltacbins, fenvbins, halo_mass_func, p_max,
-                   logM_cut, kappa, sigma, logM1, alpha, A_s, Acent, Asat, Bcent, Bsat, ic, Nthread):
+    def _compute_ngal_qso(logMbins, deltacbins, fenvbins, halo_mass_func, 
+                   logM_cut, kappa, sigma, logM1, alpha, Acent, Asat, Bcent, Bsat, ic, Nthread):
         """
         internal helper to compute number of LRGs
         """
@@ -746,8 +784,8 @@ class AbacusHOD:
                     Mh_temp = 10**logMs[i]
                     logM_cut_temp = logM_cut + Acent * deltacs[j] + Bcent * fenvs[k]
                     M1_temp = 10**(logM1 + Asat * deltacs[j] + Bsat * fenvs[k])
-                    ncent_temp = N_cen_QSO(Mh_temp, p_max, logM_cut_temp, sigma)
-                    nsat_temp = N_sat_generic(Mh_temp, 10**logM_cut_temp, kappa, M1_temp, alpha, A_s)
+                    ncent_temp = N_cen_QSO(Mh_temp, logM_cut_temp, sigma)
+                    nsat_temp = N_sat_generic(Mh_temp, 10**logM_cut_temp, kappa, M1_temp, alpha)
                     ngal_cent += halo_mass_func[i, j, k] * ncent_temp * ic
                     ngal_sat += halo_mass_func[i, j, k] * nsat_temp * ic
         return ngal_cent, ngal_sat
@@ -786,6 +824,8 @@ class AbacusHOD:
             clustering = self.compute_wp(mock_dict, *args, **kwargs)
         elif self.clustering_type == 'multipole':
             clustering = self.compute_multipole(mock_dict, *args, **kwargs)
+        else: 
+            raise ValueError('clustering_type not implemented or not specified, use xirppi, wp, multipole')
         return clustering
     
     def compute_xirppi(self, mock_dict, rpbins, pimax, pi_bin_size, Nthread = 8):
