@@ -10,48 +10,86 @@ $ python -m abacusnbody.hod.AbacusHOD.prepare_sim --path2config /path/to/config.
 import os
 from pathlib import Path
 import yaml
-
-import numpy as np
+import multiprocessing
+from multiprocessing import Pool
 import random
 import time
+import itertools
+from itertools import repeat
+import argparse
+import gc
+
+import numpy as np
 from astropy.table import Table
 import h5py
 from scipy.ndimage import gaussian_filter
 from scipy.interpolate import NearestNDInterpolator
-from itertools import repeat
-import argparse
+from numba import njit, types, jit
+import numba
+from scipy.spatial import cKDTree
 
 from abacusnbody.data.compaso_halo_catalog import CompaSOHaloCatalog
 
-import multiprocessing
-from multiprocessing import Pool
-
-from sklearn.neighbors import KDTree
 
 DEFAULTS = {}
 DEFAULTS['path2config'] = 'config/abacus_hod.yaml'
 
 
 # https://arxiv.org/pdf/2001.06018.pdf Figure 13 shows redshift evolution of LRG HOD 
-# the subsampling curve for halos
 def subsample_halos(m, MT):
     x = np.log10(m)
+    downfactors = np.zeros(len(x))
     if MT:
-        downfactors = np.zeros(len(x))
-        mask = x < 12.07
-        downfactors[mask] = 0.2/(1.0 + 10*np.exp(-(x[mask] - 11.2)*25))
-        downfactors[~mask] = 1.0/(1.0 + 0.1*np.exp(-(x[~mask] - 12.6)*7))
+        mask1 = x < 11.4
+        mask2 = x < 11.6
+        downfactors[mask1] = 0.2/(1.0 + 10*np.exp(-(x[mask1] - 11.2)*25))
+        downfactors[mask2&(~mask1)] = 0.4/(1.0 + 10*np.exp(-(x[mask2&(~mask1)] - 11.3)*25))
+        downfactors[~mask2] = 1.0/(1.0 + 0.1*np.exp(-(x[~mask2] - 11.7)*10))
+        # save all halos that could host a satellite
+        # downfactors[x>11.2] = 1
         return downfactors
     else:
-        return 1.0/(1.0 + 0.1*np.exp(-(x - 12.6)*7)) # LRG only
+        downfactors = 1.0/(1.0 + 0.1*np.exp(-(x - 12.3)*10)) # LRG only, might be able to step back to 12.5 depending on bestfit
+        downfactors[x > 13.0] = 1
+        return downfactors
 
-def subsample_particles(m, MT):
-    x = np.log10(m)
-    # return 4/(200.0 + np.exp(-(x - 13.7)*8)) # LRG only
+# def subsample_particles(m_in, n_in, MT):
+#     x = np.log10(m_in)
+    
+#     if MT:
+#         if 10**x < 1e11:
+#             return 0.01
+#         else:
+#             # a target number of particles
+#             ntarget = 4 + 10**(x-13)/(1+np.exp(-(x-13)))# + np.log(1+np.exp(200*(x-13)))
+#             return np.minimum(0.7, ntarget / n_in)
+#     else:
+#         if 10**x < 1e12:
+#             return 0.01 # essentially removing particles in halos below Mmin
+#         else:
+#             ntarget = 2 + 1.5*10**(x-13)/(1+np.exp(-(x-13)))
+#             return np.minimum(1.0, ntarget / n_in)
+
+def submask_particles(m_in, n_in, MT):
+    x = np.log10(m_in)
+    
     if MT:
-        return 0.03 # 4/(200.0 + np.exp(-(x - 13.2)*6)) # MT
+        if 10**x < 1e11:
+            return np.zeros(n_in)
+        else:
+            # a target number of particles
+            ntarget = np.minimum(n_in, int(1 + 10**(x-13)))
+            submask = np.zeros(n_in).astype(int)
+            submask[np.random.choice(n_in, ntarget, replace = False)] = 1
+            return submask
     else:
-        return 4/(200.0 + np.exp(-(x - 13.7)*8)) # LRG only
+        if 10**x < 1e12:
+            return np.zeros(n_in) # essentially removing particles in halos below Mmin
+        else:
+            ntarget = np.minimum(n_in, int(1 + 1.5*10**(x-13)))
+            submask = np.zeros(n_in).astype(int)
+            submask[np.random.choice(n_in, ntarget, replace = False)] = 1
+            return submask
 
 # # these two functions are for grid based density calculation. We found the grid based definiton is disfavored by data
 # def get_smo_density_oneslab(i, simdir, simname, z_mock, N_dim, cleaning):
@@ -168,7 +206,87 @@ def gen_rand(N, chi_min, chi_max, fac, Lbox, offset, origins):
 
     return rands_pos, rands_chis
 
-def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ranks, want_AB, cleaning, newseed, halo_lc=False):
+def concat_to_arr(lists, dtype=np.int64):
+    '''Concatenate an iterable of lists to a flat Numpy array.
+    Returns the concatenated array and the index where each list starts.
+    '''
+    starts = np.empty(len(lists) + 1, dtype=np.int64)
+    starts[0] = 0
+    starts[1:] = np.cumsum(np.fromiter((len(l) for l in lists), count=len(lists), dtype=np.int64))
+    N = starts[-1]
+    res = np.fromiter(itertools.chain.from_iterable(lists), count=N, dtype=dtype)
+    return res, starts
+
+@njit(parallel=False)
+def calc_Menv(masses, inner_arr, inner_starts, outer_arr, outer_starts):
+    N = len(inner_starts)-1
+    Menv = np.zeros(N, dtype=np.float32)
+    for p in numba.prange(N):
+        j = inner_starts[p]
+        k = inner_starts[p+1]
+        inner_mass = np.sum(masses[inner_arr[j:k]])
+
+        j = outer_starts[p]
+        k = outer_starts[p+1]
+        outer_mass = np.sum(masses[outer_arr[j:k]])
+
+        Menv[p] = outer_mass - inner_mass
+    return Menv
+
+@njit(parallel=False)
+def calc_fenv_opt(Menv, mbins, halosM):
+    fenv_rank = np.zeros(len(Menv))
+    for ibin in numba.prange(len(mbins)-1):
+        mmask = (halosM > mbins[ibin]) & (halosM < mbins[ibin + 1])
+        Nmask = np.sum(mmask)
+        if Nmask > 1:
+            new_fenv_rank = Menv[mmask].argsort().argsort()
+            fenv_rank[mmask] = new_fenv_rank / (Nmask-1) - 0.5  # max rank is always Nmask - 1
+    return fenv_rank
+
+
+def do_Menv_from_tree(allpos, allmasses, r_inner, r_outer, halo_lc, Lbox, nthread, mcut = 1e11):
+    '''Calculate a local mass environment by taking the difference in
+    total neighbor halo mass at two apertures
+    '''
+
+    if halo_lc:
+        querypos = allpos
+        treebox = None  # periodicity not needed for halo light cones
+    else:
+        # note that periodicity exists only in y and z directions
+        querypos = (allpos+Lbox/2.) % Lbox  # needs to be within 0 and Lbox for periodicity
+        treebox = Lbox
+    
+    mmask = allmasses > mcut
+    pos_cut = querypos[mmask]
+    
+    print("Building and querying trees for mass env calculation")
+    querypos_tree = cKDTree(querypos, boxsize=treebox)
+    if isinstance(r_inner, (list, tuple, np.ndarray)):
+        r_inner = np.array(r_inner)[mmask]
+    allinds_inner = querypos_tree.query_ball_point(pos_cut, r = r_inner, n_jobs = nthread)
+    inner_arr, inner_starts = concat_to_arr(allinds_inner)  # 7 sec
+    del allinds_inner; gc.collect()
+    
+    if isinstance(r_outer, (list, tuple, np.ndarray)):
+        r_outer = np.array(r_outer)[mmask]
+    allinds_outer = querypos_tree.query_ball_point(pos_cut, r = r_outer, n_jobs = nthread)
+    del querypos, querypos_tree; gc.collect()
+    
+    outer_arr, outer_starts = concat_to_arr(allinds_outer)
+    del allinds_outer; gc.collect()
+    
+    print("starting Menv")
+    numba.set_num_threads(nthread)
+    
+    Menv = np.zeros(len(allmasses))
+    Menv[mmask] = calc_Menv(allmasses, inner_arr, inner_starts, outer_arr, outer_starts)
+    return Menv
+
+
+def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ranks, want_AB, cleaning, newseed, 
+                 halo_lc=False, nthread = 1, overwrite = 1, mcut = 1e11, rad_outer = 5.):
     outfilename_halos = savedir+'/halos_xcom_'+str(i)+'_seed'+str(newseed)+'_abacushod_oldfenv'
     outfilename_particles = savedir+'/particles_xcom_'+str(i)+'_seed'+str(newseed)+'_abacushod_oldfenv'
     print("processing slab ", i)
@@ -182,33 +300,41 @@ def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ran
 
     np.random.seed(newseed + i)
     # if file already exists, just skip
-    # if os.path.exists(outfilename_halos) \
-    # and os.path.exists(outfilename_particles):
-    #     return 0
+    overwrite = int(overwrite)
+    if (not overwrite) and (os.path.exists(outfilename_halos)) \
+    and (os.path.exists(outfilename_particles)):
+        print("files exists, skipping ", i)
+        return 0
 
     # load the halo catalog slab
     print("loading halo catalog ")
     if halo_lc:
-        slabname = simdir+simname+'/z'+str(z_mock).ljust(5, '0')+'/lc_halo_info.asdf'
+        slabname = simdir+'/'+simname+'/z'+str(z_mock).ljust(5, '0')+'/lc_halo_info.asdf'
         id_key = 'index_halo'
         pos_key = 'pos_interp'
+        vel_key = 'vel_interp'
+        N_key = 'N_interp'
     else:
-        slabname = simdir+simname+'/halos/z'+str(z_mock).ljust(5, '0')\
+        slabname = simdir+'/'+simname+'/halos/z'+str(z_mock).ljust(5, '0')\
                    +'/halo_info/halo_info_'+str(i).zfill(3)+'.asdf'
         id_key = 'id'
         pos_key = 'x_L2com'
+        vel_key = 'v_L2com'
+        N_key = 'N'
 
-    cat = CompaSOHaloCatalog(slabname, subsamples=dict(A=True, rv=True), fields = ['N', 
-        pos_key, 'v_L2com', 'r90_L2com', 'r25_L2com', 'r98_L2com', 'npstartA', 'npoutA', id_key, 'sigmav3d_L2com'], 
+    cat = CompaSOHaloCatalog(slabname, subsamples=dict(A=True, rv=True), fields = [N_key, 
+        pos_key, vel_key, 'r90_L2com', 'r25_L2com', 'r98_L2com', 'npstartA', 'npoutA', id_key, 'sigmav3d_L2com'], 
         cleaned = cleaning)
     assert halo_lc == cat.halo_lc
     
     halos = cat.halos
-    if cleaning:
-        halos = halos[halos['N'] > 0]
     if halo_lc:
         halos['id'] = halos[id_key]
         halos['x_L2com'] = halos[pos_key]
+        halos['v_L2com'] = halos[vel_key]
+        halos['N'] = halos[N_key]
+    if cleaning:
+        halos = halos[halos['N'] > 0]
         
     parts = cat.subsamples
     header = cat.header
@@ -229,8 +355,7 @@ def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ran
     # only generate fenv ranks and c ranks if the user wants to enable secondary biases
     if want_AB:
         nbins = 100
-        mbins = np.logspace(np.log10(3e10), 15.5, nbins + 1)
-        rad_outer = 5. # TODO: maybe move to yaml file
+        mbins = np.logspace(np.log10(mcut), 15.5, nbins + 1)
 
         # # grid based environment calculation
         # dens_grid = np.array(h5py.File(savedir+"/density_field.h5", 'r')['dens'])
@@ -292,9 +417,9 @@ def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ran
                 if randpos.shape[0] > 0:
                     # random points on the edges
                     rand_N = randpos.shape[0]
-                    randpos_tree = KDTree(randpos) # TODO: needs to be periodic, fix bug
-                    randinds_inner = randpos_tree.query_radius(allpos[index_bounds], r = halos['r98_L2com'][index_bounds])
-                    randinds_outer = randpos_tree.query_radius(allpos[index_bounds], r = rad_outer)
+                    randpos_tree = cKDTree(randpos)
+                    randinds_inner = randpos_tree.query_ball_point(allpos[index_bounds], r = halos['r98_L2com'][index_bounds], n_jobs = nthread)
+                    randinds_outer = randpos_tree.query_ball_point(allpos[index_bounds], r = rad_outer, n_jobs = nthread)
                     rand_norm = np.zeros(len(index_bounds))
                     for ind in np.arange(len(index_bounds)):
                         rand_norm[ind] = (len(randinds_outer[ind]) - len(randinds_inner[ind]))
@@ -302,35 +427,36 @@ def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ran
                 else:
                     rand_norm = np.ones(len(index_bounds))
 
-        allpos_tree = KDTree(allpos)
-        allinds_inner = allpos_tree.query_radius(allpos, r = halos['r98_L2com'])
-        allinds_outer = allpos_tree.query_radius(allpos, r = rad_outer)
-        print("computing m stacks")
-        Menv = np.array([np.sum(allmasses[allinds_outer[ind]]) - np.sum(allmasses[allinds_inner[ind]]) \
-            for ind in np.arange(len(halos))])
+        Menv = do_Menv_from_tree(allpos, allmasses, r_inner=halos['r98_L2com'], r_outer=rad_outer,
+                                    halo_lc=halo_lc, Lbox=Lbox, nthread=nthread, mcut = mcut)
+        gc.collect()
+        
+        # Menv = np.array([np.sum(allmasses[allinds_outer[ind]]) - np.sum(allmasses[allinds_inner[ind]]) \
+        #     for ind in np.arange(len(halos))])
+        # Menv = calc_Menv(allmasses, allinds_outer, allinds_inner)
 
         if halo_lc and len(index_bounds) > 0:
             Menv[index_bounds] *= rand_norm
         
-        fenv_rank = np.zeros(len(Menv))
-        for ibin in range(nbins):
-            mmask = (halos['N']*Mpart > mbins[ibin]) \
-            & (halos['N']*Mpart < mbins[ibin + 1])
-            if np.sum(mmask) > 0:
-                if np.sum(mmask) == 1:
-                    fenv_rank[mmask] = 0
-                else:
-                    new_fenv_rank = Menv[mmask].argsort().argsort()
-                    fenv_rank[mmask] = new_fenv_rank / np.max(new_fenv_rank) - 0.5
+        # fenv_rank = np.zeros(len(Menv))
+        # for ibin in range(nbins):
+        #     mmask = (halos['N']*Mpart > mbins[ibin]) \
+        #     & (halos['N']*Mpart < mbins[ibin + 1])
+        #     if np.sum(mmask) > 0:
+        #         if np.sum(mmask) == 1:
+        #             fenv_rank[mmask] = 0
+        #         else:
+        #             new_fenv_rank = Menv[mmask].argsort().argsort()
+        #             fenv_rank[mmask] = new_fenv_rank / np.max(new_fenv_rank) - 0.5
 
-        halos['fenv_rank'] = fenv_rank
+        halos['fenv_rank'] = calc_fenv_opt(Menv, mbins, allmasses)
 
         # compute delta concentration
         print("computing c rank")
         halos_c = halos['r90_L2com']/halos['r25_L2com']
         deltac_rank = np.zeros(len(halos))
         for ibin in range(nbins):
-            mmask = (halos['N']*Mpart > mbins[ibin]) & (halos['N']*Mpart < mbins[ibin + 1])
+            mmask = (allmasses > mbins[ibin]) & (allmasses < mbins[ibin + 1])
             if np.sum(mmask) > 0:
                 if np.sum(mmask) == 1:
                     deltac_rank[mmask] = 0
@@ -369,13 +495,15 @@ def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ran
 
     print("compiling particle subsamples")
     start_tracker = 0
+    print(len(halos), np.sum(mask_halos))
     for j in np.arange(len(halos)):
         if j % 10000 == 0:
             print("halo id", j, end = '\r')
-        if mask_halos[j]:
-            # updating the mask tagging the particles we want to preserve
-            subsample_factor = subsample_particles(halos['N'][j] * Mpart, MT)
-            submask = np.random.binomial(n = 1, p = subsample_factor, size = halos_pnum[j])
+        if mask_halos[j] and halos['npoutA'][j] > 0:
+            # subsample_factor = subsample_particles(halos['N'][j] * Mpart, halos['npoutA'][j], MT)
+            # submask = np.random.binomial(n = 1, p = subsample_factor, size = halos_pnum[j])
+            submask = submask_particles(halos['N'][j] * Mpart, halos['npoutA'][j], MT)
+
             # updating the particles' masks, downsample factors, halo mass
             mask_parts[halos_pstart[j]: halos_pstart[j] + halos_pnum[j]] = submask
             # print(j, halos_pstart, halos_pnum, p_halos, downsample_parts)
@@ -480,7 +608,6 @@ def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ran
     # output_dir = savedir+'/halos_xcom_'+str(i)+'_seed'+str(newseed)+'_abacushodMT_new.h5'
     if os.path.exists(outfilename_halos):
         os.remove(outfilename_halos)
-    print(outfilename_halos, outfilename_particles)
     newfile = h5py.File(outfilename_halos, 'w')
     dataset = newfile.create_dataset('halos', data = halos[mask_halos])
     newfile.close()
@@ -516,7 +643,7 @@ def prepare_slab(i, savedir, simdir, simname, z_mock, tracer_flags, MT, want_ran
 
     print("pre process particle number ", len_old, " post process particle number ", len(parts))
 
-def main(path2config, params = None, alt_simname = None, newseed = 600, halo_lc = False):
+def main(path2config, params = None, alt_simname = None, alt_z = None, newseed = 600, halo_lc = False, overwrite = 1):
     print("compiling compaso halo catalogs into subsampled catalogs")
 
     config = yaml.safe_load(open(path2config))
@@ -525,7 +652,9 @@ def main(path2config, params = None, alt_simname = None, newseed = 600, halo_lc
         config.update(params)
     if alt_simname: 
         config['sim_params']['sim_name'] = alt_simname
-
+    if alt_z: 
+        config['sim_params']['z_mock'] = alt_z
+    
     simname = config['sim_params']['sim_name'] # "AbacusSummit_base_c000_ph006"
     simdir = config['sim_params']['sim_dir']
     z_mock = config['sim_params']['z_mock']
@@ -537,9 +666,12 @@ def main(path2config, params = None, alt_simname = None, newseed = 600, halo_lc
     if halo_lc:
         halo_info_fns = [str(Path(simdir) / Path(simname) / ('z%4.3f'%z_mock) / 'lc_halo_info.asdf')]
     else:
-        halo_info_fns = list((Path(simdir) / Path(simname) / 'halos' / ('z%4.3f'%z_mock) / 'halo_info').glob('*.asdf'))
+        halo_info_fns = list(sorted((Path(simdir) / Path(simname) / 'halos' / ('z%4.3f'%z_mock) / 'halo_info').glob('*.asdf')))
     numslabs = len(halo_info_fns)
 
+    if numslabs == 0:
+        raise ValueError('prepare_sim could not find any slabs!')
+
     tracer_flags = config['HOD_params']['tracer_flags']
     MT = False
     if tracer_flags['ELG'] or tracer_flags['QSO']:
@@ -547,6 +679,7 @@ def main(path2config, params = None, alt_simname = None, newseed = 600, halo_lc
     want_ranks = config['HOD_params']['want_ranks']
     want_AB = config['HOD_params']['want_AB']
     # N_dim = config['HOD_params']['Ndim']
+    nthread = int(np.floor(multiprocessing.cpu_count()/config['prepare_sim']['Nparallel_load']))
 
     os.makedirs(savedir, exist_ok = True)
     
@@ -554,7 +687,8 @@ def main(path2config, params = None, alt_simname = None, newseed = 600, halo_lc
     p.starmap(prepare_slab, zip(range(numslabs), repeat(savedir), 
                                 repeat(simdir), repeat(simname), repeat(z_mock), 
                                 repeat(tracer_flags), repeat(MT), repeat(want_ranks), 
-                                repeat(want_AB), repeat(cleaning), repeat(newseed), repeat(halo_lc)))
+                                repeat(want_AB), repeat(cleaning), repeat(newseed), repeat(halo_lc), 
+                                repeat(nthread), repeat(overwrite)))
     p.close()
     p.join()
 
@@ -571,47 +705,22 @@ if __name__ == "__main__":
     parser.add_argument('--alt_simname',
                         help='alternative simname to process, like "AbacusSummit_base_c000_ph003"',
                        )
+    parser.add_argument('--alt_z',
+                        help='alternative z to process, like "0.8"',
+                       )
     parser.add_argument('--newseed',
                         help='alternative random number seed, positive integer', default = 600
                        )
+    parser.add_argument('--overwrite',
+                        help='overwrite existing subsamples', default = 1
+                       )
     args = vars(parser.parse_args())
 
     main(**args)
 
     print("done")
-    # # run a series of simulations
-    # param_dict = {
-    # 'sim_params' :
-    #     {
-    #     'sim_name': 'AbacusSummit_base_c000_ph006',                                 # which simulation 
-    #     # 'sim_dir': '/mnt/gosling2/bigsims/',                                        # where is the simulation
-    #     'sim_dir': '/mnt/marvin1/syuan/scratch/bigsims/',                                        # where is the simulation
-    #     'output_dir': '/mnt/marvin1/syuan/scratch/data_mocks_georgios',          # where to output galaxy mocks
-    #     'subsample_dir': '/mnt/marvin1/syuan/scratch/data_summit/',                 # where to output subsample data
-    #     'z_mock': 0.5,                                                             # which redshift slice
-    #     'Nthread_load': 7,                                                          # number of thread for organizing simulation outputs (prepare_sim)
-    #     'cleaned_halos': False
-    #     }
-    # }
-    # for i in range(25):
-    #     param_dict['sim_params']['sim_name'] = 'AbacusSummit_base_c000_ph'+str(i).zfill(3)
-    #     main(**args, params = param_dict)
 
-    # other_cosmologies = [
-    # 'AbacusSummit_base_c100_ph000',
-    # 'AbacusSummit_base_c101_ph000',
-    # 'AbacusSummit_base_c102_ph000',
-    # 'AbacusSummit_base_c103_ph000',
-    # 'AbacusSummit_base_c112_ph000',
-    # 'AbacusSummit_base_c113_ph000',
-    # 'AbacusSummit_base_c104_ph000',
-    # 'AbacusSummit_base_c105_ph000',
-    # 'AbacusSummit_base_c108_ph000',
-    # 'AbacusSummit_base_c109_ph000',
-    # 'AbacusSummit_base_c009_ph000'
-    # ]
- 
-    # for ecosmo in other_cosmologies:
-    #     print(ecosmo)
-    #     param_dict['sim_params']['sim_name'] = ecosmo
-    #     main(**args, params = param_dict)
+    # for i in [3,4]: # [20, 21, 22, 23, 24, 0, 1, 2]: 
+    #     simname = 'AbacusSummit_base_c000_ph'+str(i).zfill(3)
+    #     for z in [0.575]:
+    #         main(args['path2config'], alt_simname = simname, alt_z = z)
